// fully unrolled implementation of SHA-1 (2-bit toggle)
// Stanislaw K Skowronek, 2007

module sha1_r1(clk, vi, wi, xi, yi, zi, k, vo, wo, xo, yo, zo);
input clk;
input [31:0] vi, wi, xi, yi, zi;
input [31:0] k;
output [31:0] vo, wo, xo, yo, zo;

wire [31:0] f = (wi&(xi^yi))^yi;
wire [31:0] vr = {vi[26:0], vi[31:27]};
wire [31:0] wr = {wi[1:0], wi[31:2]};
wire [31:0] ps4 = zi + f + k + 32'h5A827999 + vr;
reg [31:0] vo, wo, xo, yo, zo;

always @(posedge clk) begin
    vo <= vi;
    wo <= wr;
    xo <= xi;
    yo <= yi;
    zo <= ps4;
end
endmodule

module sha1_r2(clk, vi, wi, xi, yi, zi, k, vo, wo, xo, yo, zo);
input clk;
input [31:0] vi, wi, xi, yi, zi;
input [31:0] k;
output [31:0] vo, wo, xo, yo, zo;

wire [31:0] f = wi^xi^yi;
wire [31:0] vr = {vi[26:0], vi[31:27]};
wire [31:0] wr = {wi[1:0], wi[31:2]};
wire [31:0] ps1 = zi + f;
wire [31:0] ps2 = k + 32'h6ED9EBA1;
wire [31:0] ps3 = ps2 + vr;
wire [31:0] ps4 = ps1 + ps3;
reg [31:0] vo, wo, xo, yo, zo;

always @(posedge clk) begin
    vo <= vi;
    wo <= wr;
    xo <= xi;
    yo <= yi;
    zo <= ps4;
end
endmodule

module sha1_r3(clk, vi, wi, xi, yi, zi, k, vo, wo, xo, yo, zo);
input clk;
input [31:0] vi, wi, xi, yi, zi;
input [31:0] k;
output [31:0] vo, wo, xo, yo, zo;

wire [31:0] f = ((wi|xi)&yi)|(wi&xi);
wire [31:0] vr = {vi[26:0], vi[31:27]};
wire [31:0] wr = {wi[1:0], wi[31:2]};
wire [31:0] ps1 = zi + f;
wire [31:0] ps2 = k + 32'h8F1BBCDC;
wire [31:0] ps3 = ps2 + vr;
wire [31:0] ps4 = ps1 + ps3;
reg [31:0] vo, wo, xo, yo, zo;

always @(posedge clk) begin
    vo <= vi;
    wo <= wr;
    xo <= xi;
    yo <= yi;
    zo <= ps4;
end
endmodule

module sha1_r4(clk, vi, wi, xi, yi, zi, k, vo, wo, xo, yo, zo);
input clk;
input [31:0] vi, wi, xi, yi, zi;
input [31:0] k;
output [31:0] vo, wo, xo, yo, zo;

wire [31:0] f = wi^xi^yi;
wire [31:0] vr = {vi[26:0], vi[31:27]};
wire [31:0] wr = {wi[1:0], wi[31:2]};
wire [31:0] ps1 = zi + f;
wire [31:0] ps2 = k + 32'hCA62C1D6;
wire [31:0] ps3 = ps2 + vr;
wire [31:0] ps4 = ps1 + ps3;
reg [31:0] vo, wo, xo, yo, zo;

always @(posedge clk) begin
    vo <= vi;
    wo <= wr;
    xo <= xi;
    yo <= yi;
    zo <= ps4;
end
endmodule

// p1t has to be >= p2t (p2t can't be set without p1t)
module sha1hash(clk, rst, p1t, p1b, p2t, p2b, hash);
input clk;
input rst;
input p1t;
input [8:0] p1b;
input p2t;
input [8:0] p2b;
output [31:0] hash;

wire p2tZ = p2t || !p1t;
wire [8:0] p1bZ = p1t ? {p1b ^ 9'h018} : 9'h000;
wire [8:0] p2bZ = p2t ? {p2b ^ 9'h018} : 9'h000;

wire [31:0] a0 = 32'h67452301;
wire [31:0] b0 = 32'hEFCDAB89;
wire [31:0] c0 = 32'h98BADCFE;
wire [31:0] d0 = 32'h10325476;
wire [31:0] e0 = 32'hC3D2E1F0;

wire [31:0] a1, b1, c1, d1, e1;
wire [31:0] a2, b2, c2, d2, e2;
wire [31:0] a3, b3, c3, d3, e3;
wire [31:0] a4, b4, c4, d4, e4;
wire [31:0] a5, b5, c5, d5, e5;
wire [31:0] a6, b6, c6, d6, e6;
wire [31:0] a7, b7, c7, d7, e7;
wire [31:0] a8, b8, c8, d8, e8;
wire [31:0] a9, b9, c9, d9, e9;
wire [31:0] a10, b10, c10, d10, e10;
wire [31:0] a11, b11, c11, d11, e11;
wire [31:0] a12, b12, c12, d12, e12;
wire [31:0] a13, b13, c13, d13, e13;
wire [31:0] a14, b14, c14, d14, e14;
wire [31:0] a15, b15, c15, d15, e15;
wire [31:0] a16, b16, c16, d16, e16;
wire [31:0] a17, b17, c17, d17, e17;
wire [31:0] a18, b18, c18, d18, e18;
wire [31:0] a19, b19, c19, d19, e19;
wire [31:0] a20, b20, c20, d20, e20;
wire [31:0] a21, b21, c21, d21, e21;
wire [31:0] a22, b22, c22, d22, e22;
wire [31:0] a23, b23, c23, d23, e23;
wire [31:0] a24, b24, c24, d24, e24;
wire [31:0] a25, b25, c25, d25, e25;
wire [31:0] a26, b26, c26, d26, e26;
wire [31:0] a27, b27, c27, d27, e27;
wire [31:0] a28, b28, c28, d28, e28;
wire [31:0] a29, b29, c29, d29, e29;
wire [31:0] a30, b30, c30, d30, e30;
wire [31:0] a31, b31, c31, d31, e31;
wire [31:0] a32, b32, c32, d32, e32;
wire [31:0] a33, b33, c33, d33, e33;
wire [31:0] a34, b34, c34, d34, e34;
wire [31:0] a35, b35, c35, d35, e35;
wire [31:0] a36, b36, c36, d36, e36;
wire [31:0] a37, b37, c37, d37, e37;
wire [31:0] a38, b38, c38, d38, e38;
wire [31:0] a39, b39, c39, d39, e39;
wire [31:0] a40, b40, c40, d40, e40;
wire [31:0] a41, b41, c41, d41, e41;
wire [31:0] a42, b42, c42, d42, e42;
wire [31:0] a43, b43, c43, d43, e43;
wire [31:0] a44, b44, c44, d44, e44;
wire [31:0] a45, b45, c45, d45, e45;
wire [31:0] a46, b46, c46, d46, e46;
wire [31:0] a47, b47, c47, d47, e47;
wire [31:0] a48, b48, c48, d48, e48;
wire [31:0] a49, b49, c49, d49, e49;
wire [31:0] a50, b50, c50, d50, e50;
wire [31:0] a51, b51, c51, d51, e51;
wire [31:0] a52, b52, c52, d52, e52;
wire [31:0] a53, b53, c53, d53, e53;
wire [31:0] a54, b54, c54, d54, e54;
wire [31:0] a55, b55, c55, d55, e55;
wire [31:0] a56, b56, c56, d56, e56;
wire [31:0] a57, b57, c57, d57, e57;
wire [31:0] a58, b58, c58, d58, e58;
wire [31:0] a59, b59, c59, d59, e59;
wire [31:0] a60, b60, c60, d60, e60;
wire [31:0] a61, b61, c61, d61, e61;
wire [31:0] a62, b62, c62, d62, e62;
wire [31:0] a63, b63, c63, d63, e63;
wire [31:0] a64, b64, c64, d64, e64;
wire [31:0] a65, b65, c65, d65, e65;
wire [31:0] a66, b66, c66, d66, e66;
wire [31:0] a67, b67, c67, d67, e67;
wire [31:0] a68, b68, c68, d68, e68;
wire [31:0] a69, b69, c69, d69, e69;
wire [31:0] a70, b70, c70, d70, e70;
wire [31:0] a71, b71, c71, d71, e71;
wire [31:0] a72, b72, c72, d72, e72;
wire [31:0] a73, b73, c73, d73, e73;
wire [31:0] a74, b74, c74, d74, e74;
wire [31:0] a75, b75, c75, d75, e75;
wire [31:0] a76, b76, c76, d76, e76;
wire [31:0] a77, b77, c77, d77, e77;
wire [31:0] a78, b78, c78, d78, e78;
wire [31:0] a79, b79, c79, d79, e79;
wire [31:0] a80, b80, c80, d80, e80;

wire [31:0] k0, k1, k2, k3, k4, k5, k6, k7;
wire [31:0] k8, k9, k10, k11, k12, k13, k14, k15;
wire [31:0] k16, k17, k18, k19, k20, k21, k22, k23;
wire [31:0] k24, k25, k26, k27, k28, k29, k30, k31;
wire [31:0] k32, k33, k34, k35, k36, k37, k38, k39;
wire [31:0] k40, k41, k42, k43, k44, k45, k46, k47;
wire [31:0] k48, k49, k50, k51, k52, k53, k54, k55;
wire [31:0] k56, k57, k58, k59, k60, k61, k62, k63;
wire [31:0] k64, k65, k66, k67, k68, k69, k70, k71;
wire [31:0] k72, k73, k74, k75, k76, k77, k78, k79;

wire rst0, p2t0, rst1, p2t1, rst2, p2t2, rst3, p2t3;
wire rst4, p2t4, rst5, p2t5, rst6, p2t6, rst7, p2t7;
wire rst8, p2t8, rst9, p2t9, rst10, p2t10, rst11, p2t11;
wire rst12, p2t12, rst13, p2t13, rst14, p2t14, rst15, p2t15;
wire rst16, p2t16, rst17, p2t17, rst18, p2t18, rst19, p2t19;
wire rst20, p2t20, rst21, p2t21, rst22, p2t22, rst23, p2t23;
wire rst24, p2t24, rst25, p2t25, rst26, p2t26, rst27, p2t27;
wire rst28, p2t28, rst29, p2t29, rst30, p2t30, rst31, p2t31;
wire rst32, p2t32, rst33, p2t33, rst34, p2t34, rst35, p2t35;
wire rst36, p2t36, rst37, p2t37, rst38, p2t38, rst39, p2t39;
wire rst40, p2t40, rst41, p2t41, rst42, p2t42, rst43, p2t43;
wire rst44, p2t44, rst45, p2t45, rst46, p2t46, rst47, p2t47;
wire rst48, p2t48, rst49, p2t49, rst50, p2t50, rst51, p2t51;
wire rst52, p2t52, rst53, p2t53, rst54, p2t54, rst55, p2t55;
wire rst56, p2t56, rst57, p2t57, rst58, p2t58, rst59, p2t59;
wire rst60, p2t60, rst61, p2t61, rst62, p2t62, rst63, p2t63;
wire rst64, p2t64, rst65, p2t65, rst66, p2t66, rst67, p2t67;
wire rst68, p2t68, rst69, p2t69, rst70, p2t70, rst71, p2t71;
wire rst72, p2t72, rst73, p2t73, rst74, p2t74, rst75, p2t75;
wire rst76, p2t76, rst77, p2t77, rst78, p2t78, rst79, p2t79;
wire [8:0] p1b0, p2b0, p1b1, p2b1, p1b2, p2b2, p1b3, p2b3;
wire [8:0] p1b4, p2b4, p1b5, p2b5, p1b6, p2b6, p1b7, p2b7;
wire [8:0] p1b8, p2b8, p1b9, p2b9, p1b10, p2b10, p1b11, p2b11;
wire [8:0] p1b12, p2b12, p1b13, p2b13, p1b14, p2b14, p1b15, p2b15;
wire [8:0] p1b16, p2b16, p1b17, p2b17, p1b18, p2b18, p1b19, p2b19;
wire [8:0] p1b20, p2b20, p1b21, p2b21, p1b22, p2b22, p1b23, p2b23;
wire [8:0] p1b24, p2b24, p1b25, p2b25, p1b26, p2b26, p1b27, p2b27;
wire [8:0] p1b28, p2b28, p1b29, p2b29, p1b30, p2b30, p1b31, p2b31;
wire [8:0] p1b32, p2b32, p1b33, p2b33, p1b34, p2b34, p1b35, p2b35;
wire [8:0] p1b36, p2b36, p1b37, p2b37, p1b38, p2b38, p1b39, p2b39;
wire [8:0] p1b40, p2b40, p1b41, p2b41, p1b42, p2b42, p1b43, p2b43;
wire [8:0] p1b44, p2b44, p1b45, p2b45, p1b46, p2b46, p1b47, p2b47;
wire [8:0] p1b48, p2b48, p1b49, p2b49, p1b50, p2b50, p1b51, p2b51;
wire [8:0] p1b52, p2b52, p1b53, p2b53, p1b54, p2b54, p1b55, p2b55;
wire [8:0] p1b56, p2b56, p1b57, p2b57, p1b58, p2b58, p1b59, p2b59;
wire [8:0] p1b60, p2b60, p1b61, p2b61, p1b62, p2b62, p1b63, p2b63;
wire [8:0] p1b64, p2b64, p1b65, p2b65, p1b66, p2b66, p1b67, p2b67;
wire [8:0] p1b68, p2b68, p1b69, p2b69, p1b70, p2b70, p1b71, p2b71;
wire [8:0] p1b72, p2b72, p1b73, p2b73, p1b74, p2b74, p1b75, p2b75;
wire [8:0] p1b76, p2b76, p1b77, p2b77, p1b78, p2b78, p1b79, p2b79;

sha1_k0 key0 (clk, rst, p1bZ, p2tZ, p2bZ, k0, rst0, p1b0, p2t0, p2b0);
sha1_k1 key1 (clk, rst0, p1b0, p2t0, p2b0, k1, rst1, p1b1, p2t1, p2b1);
sha1_k2 key2 (clk, rst1, p1b1, p2t1, p2b1, k2, rst2, p1b2, p2t2, p2b2);
sha1_k3 key3 (clk, rst2, p1b2, p2t2, p2b2, k3, rst3, p1b3, p2t3, p2b3);
sha1_k4 key4 (clk, rst3, p1b3, p2t3, p2b3, k4, rst4, p1b4, p2t4, p2b4);
sha1_k5 key5 (clk, rst4, p1b4, p2t4, p2b4, k5, rst5, p1b5, p2t5, p2b5);
sha1_k6 key6 (clk, rst5, p1b5, p2t5, p2b5, k6, rst6, p1b6, p2t6, p2b6);
sha1_k7 key7 (clk, rst6, p1b6, p2t6, p2b6, k7, rst7, p1b7, p2t7, p2b7);
sha1_k8 key8 (clk, rst7, p1b7, p2t7, p2b7, k8, rst8, p1b8, p2t8, p2b8);
sha1_k9 key9 (clk, rst8, p1b8, p2t8, p2b8, k9, rst9, p1b9, p2t9, p2b9);
sha1_k10 key10 (clk, rst9, p1b9, p2t9, p2b9, k10, rst10, p1b10, p2t10, p2b10);
sha1_k11 key11 (clk, rst10, p1b10, p2t10, p2b10, k11, rst11, p1b11, p2t11, p2b11);
sha1_k12 key12 (clk, rst11, p1b11, p2t11, p2b11, k12, rst12, p1b12, p2t12, p2b12);
sha1_k13 key13 (clk, rst12, p1b12, p2t12, p2b12, k13, rst13, p1b13, p2t13, p2b13);
sha1_k14 key14 (clk, rst13, p1b13, p2t13, p2b13, k14, rst14, p1b14, p2t14, p2b14);
sha1_k15 key15 (clk, rst14, p1b14, p2t14, p2b14, k15, rst15, p1b15, p2t15, p2b15);
sha1_k16 key16 (clk, rst15, p1b15, p2t15, p2b15, k16, rst16, p1b16, p2t16, p2b16);
sha1_k17 key17 (clk, rst16, p1b16, p2t16, p2b16, k17, rst17, p1b17, p2t17, p2b17);
sha1_k18 key18 (clk, rst17, p1b17, p2t17, p2b17, k18, rst18, p1b18, p2t18, p2b18);
sha1_k19 key19 (clk, rst18, p1b18, p2t18, p2b18, k19, rst19, p1b19, p2t19, p2b19);
sha1_k20 key20 (clk, rst19, p1b19, p2t19, p2b19, k20, rst20, p1b20, p2t20, p2b20);
sha1_k21 key21 (clk, rst20, p1b20, p2t20, p2b20, k21, rst21, p1b21, p2t21, p2b21);
sha1_k22 key22 (clk, rst21, p1b21, p2t21, p2b21, k22, rst22, p1b22, p2t22, p2b22);
sha1_k23 key23 (clk, rst22, p1b22, p2t22, p2b22, k23, rst23, p1b23, p2t23, p2b23);
sha1_k24 key24 (clk, rst23, p1b23, p2t23, p2b23, k24, rst24, p1b24, p2t24, p2b24);
sha1_k25 key25 (clk, rst24, p1b24, p2t24, p2b24, k25, rst25, p1b25, p2t25, p2b25);
sha1_k26 key26 (clk, rst25, p1b25, p2t25, p2b25, k26, rst26, p1b26, p2t26, p2b26);
sha1_k27 key27 (clk, rst26, p1b26, p2t26, p2b26, k27, rst27, p1b27, p2t27, p2b27);
sha1_k28 key28 (clk, rst27, p1b27, p2t27, p2b27, k28, rst28, p1b28, p2t28, p2b28);
sha1_k29 key29 (clk, rst28, p1b28, p2t28, p2b28, k29, rst29, p1b29, p2t29, p2b29);
sha1_k30 key30 (clk, rst29, p1b29, p2t29, p2b29, k30, rst30, p1b30, p2t30, p2b30);
sha1_k31 key31 (clk, rst30, p1b30, p2t30, p2b30, k31, rst31, p1b31, p2t31, p2b31);
sha1_k32 key32 (clk, rst31, p1b31, p2t31, p2b31, k32, rst32, p1b32, p2t32, p2b32);
sha1_k33 key33 (clk, rst32, p1b32, p2t32, p2b32, k33, rst33, p1b33, p2t33, p2b33);
sha1_k34 key34 (clk, rst33, p1b33, p2t33, p2b33, k34, rst34, p1b34, p2t34, p2b34);
sha1_k35 key35 (clk, rst34, p1b34, p2t34, p2b34, k35, rst35, p1b35, p2t35, p2b35);
sha1_k36 key36 (clk, rst35, p1b35, p2t35, p2b35, k36, rst36, p1b36, p2t36, p2b36);
sha1_k37 key37 (clk, rst36, p1b36, p2t36, p2b36, k37, rst37, p1b37, p2t37, p2b37);
sha1_k38 key38 (clk, rst37, p1b37, p2t37, p2b37, k38, rst38, p1b38, p2t38, p2b38);
sha1_k39 key39 (clk, rst38, p1b38, p2t38, p2b38, k39, rst39, p1b39, p2t39, p2b39);
sha1_k40 key40 (clk, rst39, p1b39, p2t39, p2b39, k40, rst40, p1b40, p2t40, p2b40);
sha1_k41 key41 (clk, rst40, p1b40, p2t40, p2b40, k41, rst41, p1b41, p2t41, p2b41);
sha1_k42 key42 (clk, rst41, p1b41, p2t41, p2b41, k42, rst42, p1b42, p2t42, p2b42);
sha1_k43 key43 (clk, rst42, p1b42, p2t42, p2b42, k43, rst43, p1b43, p2t43, p2b43);
sha1_k44 key44 (clk, rst43, p1b43, p2t43, p2b43, k44, rst44, p1b44, p2t44, p2b44);
sha1_k45 key45 (clk, rst44, p1b44, p2t44, p2b44, k45, rst45, p1b45, p2t45, p2b45);
sha1_k46 key46 (clk, rst45, p1b45, p2t45, p2b45, k46, rst46, p1b46, p2t46, p2b46);
sha1_k47 key47 (clk, rst46, p1b46, p2t46, p2b46, k47, rst47, p1b47, p2t47, p2b47);
sha1_k48 key48 (clk, rst47, p1b47, p2t47, p2b47, k48, rst48, p1b48, p2t48, p2b48);
sha1_k49 key49 (clk, rst48, p1b48, p2t48, p2b48, k49, rst49, p1b49, p2t49, p2b49);
sha1_k50 key50 (clk, rst49, p1b49, p2t49, p2b49, k50, rst50, p1b50, p2t50, p2b50);
sha1_k51 key51 (clk, rst50, p1b50, p2t50, p2b50, k51, rst51, p1b51, p2t51, p2b51);
sha1_k52 key52 (clk, rst51, p1b51, p2t51, p2b51, k52, rst52, p1b52, p2t52, p2b52);
sha1_k53 key53 (clk, rst52, p1b52, p2t52, p2b52, k53, rst53, p1b53, p2t53, p2b53);
sha1_k54 key54 (clk, rst53, p1b53, p2t53, p2b53, k54, rst54, p1b54, p2t54, p2b54);
sha1_k55 key55 (clk, rst54, p1b54, p2t54, p2b54, k55, rst55, p1b55, p2t55, p2b55);
sha1_k56 key56 (clk, rst55, p1b55, p2t55, p2b55, k56, rst56, p1b56, p2t56, p2b56);
sha1_k57 key57 (clk, rst56, p1b56, p2t56, p2b56, k57, rst57, p1b57, p2t57, p2b57);
sha1_k58 key58 (clk, rst57, p1b57, p2t57, p2b57, k58, rst58, p1b58, p2t58, p2b58);
sha1_k59 key59 (clk, rst58, p1b58, p2t58, p2b58, k59, rst59, p1b59, p2t59, p2b59);
sha1_k60 key60 (clk, rst59, p1b59, p2t59, p2b59, k60, rst60, p1b60, p2t60, p2b60);
sha1_k61 key61 (clk, rst60, p1b60, p2t60, p2b60, k61, rst61, p1b61, p2t61, p2b61);
sha1_k62 key62 (clk, rst61, p1b61, p2t61, p2b61, k62, rst62, p1b62, p2t62, p2b62);
sha1_k63 key63 (clk, rst62, p1b62, p2t62, p2b62, k63, rst63, p1b63, p2t63, p2b63);
sha1_k64 key64 (clk, rst63, p1b63, p2t63, p2b63, k64, rst64, p1b64, p2t64, p2b64);
sha1_k65 key65 (clk, rst64, p1b64, p2t64, p2b64, k65, rst65, p1b65, p2t65, p2b65);
sha1_k66 key66 (clk, rst65, p1b65, p2t65, p2b65, k66, rst66, p1b66, p2t66, p2b66);
sha1_k67 key67 (clk, rst66, p1b66, p2t66, p2b66, k67, rst67, p1b67, p2t67, p2b67);
sha1_k68 key68 (clk, rst67, p1b67, p2t67, p2b67, k68, rst68, p1b68, p2t68, p2b68);
sha1_k69 key69 (clk, rst68, p1b68, p2t68, p2b68, k69, rst69, p1b69, p2t69, p2b69);
sha1_k70 key70 (clk, rst69, p1b69, p2t69, p2b69, k70, rst70, p1b70, p2t70, p2b70);
sha1_k71 key71 (clk, rst70, p1b70, p2t70, p2b70, k71, rst71, p1b71, p2t71, p2b71);
sha1_k72 key72 (clk, rst71, p1b71, p2t71, p2b71, k72, rst72, p1b72, p2t72, p2b72);
sha1_k73 key73 (clk, rst72, p1b72, p2t72, p2b72, k73, rst73, p1b73, p2t73, p2b73);
sha1_k74 key74 (clk, rst73, p1b73, p2t73, p2b73, k74, rst74, p1b74, p2t74, p2b74);
sha1_k75 key75 (clk, rst74, p1b74, p2t74, p2b74, k75, rst75, p1b75, p2t75, p2b75);
sha1_k76 key76 (clk, rst75, p1b75, p2t75, p2b75, k76, rst76, p1b76, p2t76, p2b76);
sha1_k77 key77 (clk, rst76, p1b76, p2t76, p2b76, k77, rst77, p1b77, p2t77, p2b77);
sha1_k78 key78 (clk, rst77, p1b77, p2t77, p2b77, k78, rst78, p1b78, p2t78, p2b78);
sha1_k79 key79 (clk, rst78, p1b78, p2t78, p2b78, k79, rst79, p1b79, p2t79, p2b79);

sha1_r1 round1 (clk, a0, b0, c0, d0, e0, k0, a1, b1, c1, d1, e1);
sha1_r1 round2 (clk, e1, a1, b1, c1, d1, k1, e2, a2, b2, c2, d2);
sha1_r1 round3 (clk, d2, e2, a2, b2, c2, k2, d3, e3, a3, b3, c3);
sha1_r1 round4 (clk, c3, d3, e3, a3, b3, k3, c4, d4, e4, a4, b4);
sha1_r1 round5 (clk, b4, c4, d4, e4, a4, k4, b5, c5, d5, e5, a5);
sha1_r1 round6 (clk, a5, b5, c5, d5, e5, k5, a6, b6, c6, d6, e6);
sha1_r1 round7 (clk, e6, a6, b6, c6, d6, k6, e7, a7, b7, c7, d7);
sha1_r1 round8 (clk, d7, e7, a7, b7, c7, k7, d8, e8, a8, b8, c8);
sha1_r1 round9 (clk, c8, d8, e8, a8, b8, k8, c9, d9, e9, a9, b9);
sha1_r1 round10 (clk, b9, c9, d9, e9, a9, k9, b10, c10, d10, e10, a10);
sha1_r1 round11 (clk, a10, b10, c10, d10, e10, k10, a11, b11, c11, d11, e11);
sha1_r1 round12 (clk, e11, a11, b11, c11, d11, k11, e12, a12, b12, c12, d12);
sha1_r1 round13 (clk, d12, e12, a12, b12, c12, k12, d13, e13, a13, b13, c13);
sha1_r1 round14 (clk, c13, d13, e13, a13, b13, k13, c14, d14, e14, a14, b14);
sha1_r1 round15 (clk, b14, c14, d14, e14, a14, k14, b15, c15, d15, e15, a15);
sha1_r1 round16 (clk, a15, b15, c15, d15, e15, k15, a16, b16, c16, d16, e16);
sha1_r1 round17 (clk, e16, a16, b16, c16, d16, k16, e17, a17, b17, c17, d17);
sha1_r1 round18 (clk, d17, e17, a17, b17, c17, k17, d18, e18, a18, b18, c18);
sha1_r1 round19 (clk, c18, d18, e18, a18, b18, k18, c19, d19, e19, a19, b19);
sha1_r1 round20 (clk, b19, c19, d19, e19, a19, k19, b20, c20, d20, e20, a20);
sha1_r2 round21 (clk, a20, b20, c20, d20, e20, k20, a21, b21, c21, d21, e21);
sha1_r2 round22 (clk, e21, a21, b21, c21, d21, k21, e22, a22, b22, c22, d22);
sha1_r2 round23 (clk, d22, e22, a22, b22, c22, k22, d23, e23, a23, b23, c23);
sha1_r2 round24 (clk, c23, d23, e23, a23, b23, k23, c24, d24, e24, a24, b24);
sha1_r2 round25 (clk, b24, c24, d24, e24, a24, k24, b25, c25, d25, e25, a25);
sha1_r2 round26 (clk, a25, b25, c25, d25, e25, k25, a26, b26, c26, d26, e26);
sha1_r2 round27 (clk, e26, a26, b26, c26, d26, k26, e27, a27, b27, c27, d27);
sha1_r2 round28 (clk, d27, e27, a27, b27, c27, k27, d28, e28, a28, b28, c28);
sha1_r2 round29 (clk, c28, d28, e28, a28, b28, k28, c29, d29, e29, a29, b29);
sha1_r2 round30 (clk, b29, c29, d29, e29, a29, k29, b30, c30, d30, e30, a30);
sha1_r2 round31 (clk, a30, b30, c30, d30, e30, k30, a31, b31, c31, d31, e31);
sha1_r2 round32 (clk, e31, a31, b31, c31, d31, k31, e32, a32, b32, c32, d32);
sha1_r2 round33 (clk, d32, e32, a32, b32, c32, k32, d33, e33, a33, b33, c33);
sha1_r2 round34 (clk, c33, d33, e33, a33, b33, k33, c34, d34, e34, a34, b34);
sha1_r2 round35 (clk, b34, c34, d34, e34, a34, k34, b35, c35, d35, e35, a35);
sha1_r2 round36 (clk, a35, b35, c35, d35, e35, k35, a36, b36, c36, d36, e36);
sha1_r2 round37 (clk, e36, a36, b36, c36, d36, k36, e37, a37, b37, c37, d37);
sha1_r2 round38 (clk, d37, e37, a37, b37, c37, k37, d38, e38, a38, b38, c38);
sha1_r2 round39 (clk, c38, d38, e38, a38, b38, k38, c39, d39, e39, a39, b39);
sha1_r2 round40 (clk, b39, c39, d39, e39, a39, k39, b40, c40, d40, e40, a40);
sha1_r3 round41 (clk, a40, b40, c40, d40, e40, k40, a41, b41, c41, d41, e41);
sha1_r3 round42 (clk, e41, a41, b41, c41, d41, k41, e42, a42, b42, c42, d42);
sha1_r3 round43 (clk, d42, e42, a42, b42, c42, k42, d43, e43, a43, b43, c43);
sha1_r3 round44 (clk, c43, d43, e43, a43, b43, k43, c44, d44, e44, a44, b44);
sha1_r3 round45 (clk, b44, c44, d44, e44, a44, k44, b45, c45, d45, e45, a45);
sha1_r3 round46 (clk, a45, b45, c45, d45, e45, k45, a46, b46, c46, d46, e46);
sha1_r3 round47 (clk, e46, a46, b46, c46, d46, k46, e47, a47, b47, c47, d47);
sha1_r3 round48 (clk, d47, e47, a47, b47, c47, k47, d48, e48, a48, b48, c48);
sha1_r3 round49 (clk, c48, d48, e48, a48, b48, k48, c49, d49, e49, a49, b49);
sha1_r3 round50 (clk, b49, c49, d49, e49, a49, k49, b50, c50, d50, e50, a50);
sha1_r3 round51 (clk, a50, b50, c50, d50, e50, k50, a51, b51, c51, d51, e51);
sha1_r3 round52 (clk, e51, a51, b51, c51, d51, k51, e52, a52, b52, c52, d52);
sha1_r3 round53 (clk, d52, e52, a52, b52, c52, k52, d53, e53, a53, b53, c53);
sha1_r3 round54 (clk, c53, d53, e53, a53, b53, k53, c54, d54, e54, a54, b54);
sha1_r3 round55 (clk, b54, c54, d54, e54, a54, k54, b55, c55, d55, e55, a55);
sha1_r3 round56 (clk, a55, b55, c55, d55, e55, k55, a56, b56, c56, d56, e56);
sha1_r3 round57 (clk, e56, a56, b56, c56, d56, k56, e57, a57, b57, c57, d57);
sha1_r3 round58 (clk, d57, e57, a57, b57, c57, k57, d58, e58, a58, b58, c58);
sha1_r3 round59 (clk, c58, d58, e58, a58, b58, k58, c59, d59, e59, a59, b59);
sha1_r3 round60 (clk, b59, c59, d59, e59, a59, k59, b60, c60, d60, e60, a60);
sha1_r4 round61 (clk, a60, b60, c60, d60, e60, k60, a61, b61, c61, d61, e61);
sha1_r4 round62 (clk, e61, a61, b61, c61, d61, k61, e62, a62, b62, c62, d62);
sha1_r4 round63 (clk, d62, e62, a62, b62, c62, k62, d63, e63, a63, b63, c63);
sha1_r4 round64 (clk, c63, d63, e63, a63, b63, k63, c64, d64, e64, a64, b64);
sha1_r4 round65 (clk, b64, c64, d64, e64, a64, k64, b65, c65, d65, e65, a65);
sha1_r4 round66 (clk, a65, b65, c65, d65, e65, k65, a66, b66, c66, d66, e66);
sha1_r4 round67 (clk, e66, a66, b66, c66, d66, k66, e67, a67, b67, c67, d67);
sha1_r4 round68 (clk, d67, e67, a67, b67, c67, k67, d68, e68, a68, b68, c68);
sha1_r4 round69 (clk, c68, d68, e68, a68, b68, k68, c69, d69, e69, a69, b69);
sha1_r4 round70 (clk, b69, c69, d69, e69, a69, k69, b70, c70, d70, e70, a70);
sha1_r4 round71 (clk, a70, b70, c70, d70, e70, k70, a71, b71, c71, d71, e71);
sha1_r4 round72 (clk, e71, a71, b71, c71, d71, k71, e72, a72, b72, c72, d72);
sha1_r4 round73 (clk, d72, e72, a72, b72, c72, k72, d73, e73, a73, b73, c73);
sha1_r4 round74 (clk, c73, d73, e73, a73, b73, k73, c74, d74, e74, a74, b74);
sha1_r4 round75 (clk, b74, c74, d74, e74, a74, k74, b75, c75, d75, e75, a75);
sha1_r4 round76 (clk, a75, b75, c75, d75, e75, k75, a76, b76, c76, d76, e76);
sha1_r4 round77 (clk, e76, a76, b76, c76, d76, k76, e77, a77, b77, c77, d77);
sha1_r4 round78 (clk, d77, e77, a77, b77, c77, k77, d78, e78, a78, b78, c78);
sha1_r4 round79 (clk, c78, d78, e78, a78, b78, k78, c79, d79, e79, a79, b79);
sha1_r4 round80 (clk, b79, c79, d79, e79, a79, k79, b80, c80, d80, e80, a80);

wire [31:0] ap = a80 + 32'h67452301;
reg [31:0] hash;
always @(posedge clk)
    hash <= {ap[7:0], ap[15:8], ap[23:16], ap[31:24]};

endmodule
